package io.sixminutes.ridicule

import android.app.ActivityManager
import android.content.Context
import android.content.Intent
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.provider.Settings
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.WindowManager
import androidx.activity.result.contract.ActivityResultContracts
import androidx.activity.viewModels
import androidx.appcompat.app.AppCompatActivity
import androidx.core.view.isVisible
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.lifecycleScope
import androidx.lifecycle.repeatOnLifecycle
import io.sixminutes.ridicule.databinding.ActivityMainBinding
import io.sixminutes.ridicule.databinding.FloatingWindowBinding
import kotlinx.coroutines.flow.collectLatest
import kotlinx.coroutines.launch

class MainActivity : AppCompatActivity() {
    private lateinit var binding: ActivityMainBinding
    private val viewModel: MainViewModel by viewModels()

    // 使用懒加载处理权限请求
    private val overlayPermissionLauncher by lazy {
        registerForActivityResult(ActivityResultContracts.StartActivityForResult()) {
            if (isActive()) viewModel.checkOverlayPermission(applicationContext)
        }
    }

    private val accessibilityLauncher by lazy {
        registerForActivityResult(ActivityResultContracts.StartActivityForResult()) {
            if (isActive()) viewModel.checkAccessibilityService()
        }
    }

    // 跟踪悬浮窗状态
    private var isFloatingWindowVisible = false

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityMainBinding.inflate(layoutInflater)
        setContentView(binding.root)

        // 初始化 ViewModel
        viewModel.onViewCreated() // 调用 ViewModel 的初始化方法

        setupObservers()
        setupClickListeners()
        handleDeepLink(intent)
    }

    override fun onNewIntent(intent: Intent) {
        super.onNewIntent(intent)
        handleDeepLink(intent)
    }

    override fun onDestroy() {
        super.onDestroy()
        // 确保移除悬浮窗视图
        if (isFloatingWindowVisible) {
            viewModel.getFloatingView()?.let {
                windowManager.removeView(it)
            }
        }
        viewModel.cleanUpResources()
    }

    /**
     * 设置观察者
     */
    private fun setupObservers() {
        lifecycleScope.launch {
            repeatOnLifecycle(Lifecycle.State.STARTED) {
                viewModel.state.collectLatest { state ->
                    with(state) {
                        // 更新UI状态
                        binding.progressBar.isVisible = isLoading
                        binding.gaidText.text = gaid
                        binding.launchButton.isEnabled = isServiceReady

                        // 处理悬浮窗显示/隐藏
                        floatingWindowVisible?.let { visible ->
                            when {
                                visible && !isFloatingWindowVisible -> showFloatingWindow()
                                !visible && isFloatingWindowVisible -> hideFloatingWindow()
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * 设置点击监听器
     */
    private fun setupClickListeners() {
        binding.launchButton.setOnClickListener {
            if (isActive()) {
                viewModel.requestOverlayPermission(this@MainActivity)
                viewModel.launchTargetApp()
            }
        }
    }

    /**
     * 显示悬浮窗
     */
    private fun showFloatingWindow() {
        try {
            val windowBinding = FloatingWindowBinding.inflate(LayoutInflater.from(this))
            val params = WindowManager.LayoutParams().apply {
                configureWindowParams()
                windowBinding.root.setOnClickListener {
                    if (isActive()) {
                        hideFloatingWindow()
                    }
                }
            }
            windowManager.addView(windowBinding.root, params)
            viewModel.setFloatingView(windowBinding.root)
            isFloatingWindowVisible = true
        } catch (e: Exception) {
            Log.e("MainActivity", "Show floating window failed", e)
        }
    }

    /**
     * 隐藏悬浮窗并回到主窗口
     */
    private fun hideFloatingWindow() {
        try {
            viewModel.getFloatingView()?.let {
                windowManager.removeView(it)
                isFloatingWindowVisible = false
            }

            // 强制将 MainActivity 带到前台
            bringMainActivityToFront()

            // 启动 MainActivity
            val intent = Intent(this, MainActivity::class.java).apply {
                flags = Intent.FLAG_ACTIVITY_NEW_TASK or
                        Intent.FLAG_ACTIVITY_CLEAR_TOP or
                        Intent.FLAG_ACTIVITY_REORDER_TO_FRONT or
                        Intent.FLAG_ACTIVITY_BROUGHT_TO_FRONT
            }
            startActivity(intent)
        } catch (e: Exception) {
            Log.e("MainActivity", "Hide floating window failed", e)
        }
    }

    private fun bringMainActivityToFront() {
        val activityManager = getSystemService(Context.ACTIVITY_SERVICE) as ActivityManager
        val tasks = activityManager.getRunningTasks(10) // 获取最近的任务列表
        var found = false
        for (task in tasks) {
            if (task.topActivity?.packageName == packageName) {
                // 找到 MainActivity 的任务栈，将其带到前台
                activityManager.moveTaskToFront(task.id, ActivityManager.MOVE_TASK_WITH_HOME)
                found = true
                break
            }
        }
        if (found)
            Log.e("MainActivity", "MainActivity found")
        else
            Log.e("MainActivity", "MainActivity not found")
    }

    /**
     * 处理深度链接
     * @param intent 意图
     */
    private fun handleDeepLink(intent: Intent?) {
        intent?.data?.let { uri ->
            when (uri.path) {
                "/activate" -> viewModel.checkAccessibilityService()
            }
        }
    }

    /**
     * 配置窗口参数
     */
    private fun WindowManager.LayoutParams.configureWindowParams() {
        width = WindowManager.LayoutParams.WRAP_CONTENT
        height = WindowManager.LayoutParams.WRAP_CONTENT
        type = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            WindowManager.LayoutParams.TYPE_APPLICATION_OVERLAY
        } else {
            WindowManager.LayoutParams.TYPE_PHONE
        }
        flags = WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE or
                WindowManager.LayoutParams.FLAG_LAYOUT_NO_LIMITS
        format = android.graphics.PixelFormat.TRANSLUCENT
        gravity = android.view.Gravity.BOTTOM or android.view.Gravity.END // 修改为右下角
        x = 0 // 初始 x 坐标
        y = 0 // 初始 y 坐标
    }

    /**
     * 检查Activity是否活跃
     * @return 是否活跃
     */
    private fun Context.isActive(): Boolean {
        return if (this is MainActivity) {
            !isFinishing && !isDestroyed && lifecycle.currentState.isAtLeast(Lifecycle.State.CREATED)
        } else {
            false
        }
    }
}