package io.sixminutes.ridicule

import android.app.Application
import android.content.Context
import android.content.Intent
import android.net.Uri
import android.os.Build
import android.provider.Settings
import android.util.Log
import android.view.View
import android.view.WindowManager
import androidx.lifecycle.AndroidViewModel
import androidx.lifecycle.viewModelScope
import kotlinx.coroutines.delay
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.flow.update
import kotlinx.coroutines.launch
import java.lang.ref.WeakReference

class MainViewModel(application: Application) : AndroidViewModel(application) {

    // region 状态管理
    private val _state = MutableStateFlow(MainViewState())
    val state = _state.asStateFlow()

    private data class InternalState(
        var floatingViewRef: WeakReference<View>? = null
    )

    private val internalState = InternalState()
    // endregion

    // region 依赖项
    private val gaidHelper by lazy { GAIDHelper(application) }
    private val windowManager by lazy {
        getApplication<Application>().getSystemService(Context.WINDOW_SERVICE) as WindowManager
    }
    // endregion

    // region 公共API
    fun onViewCreated() {
        fetchGAID() // 获取 GAID
        checkAccessibilityService() // 检查无障碍服务是否启用
    }

    fun checkAccessibilityService() {
        val service = MyAccessibilityService.getInstance()
        updateState { it.copy(isServiceReady = service != null) }

        if (service == null) {
            requestAccessibilityPermission()
        }
    }

    fun launchTargetApp() {
        // 首先发送一个 Intent 模拟按下 Home 键，返回到桌面
        val homeIntent = Intent(Intent.ACTION_MAIN).apply {
            addCategory(Intent.CATEGORY_HOME)
            flags = Intent.FLAG_ACTIVITY_NEW_TASK
        }
        getApplication<Application>().startActivity(homeIntent)

        viewModelScope.launch {
            delay(1000) // 延迟 1000 毫秒，确保桌面已经显示
            try {
                MyAccessibilityService.getInstance()?.findAndLaunchApp("breakingnews")
            } catch (e: Exception) {
                Log.e("ViewModel", "Launch app failed", e)
                updateState { it.copy(errorMessage = "Failed to launch app") }
            }
        }
    }

    fun requestOverlayPermission(context: Context) {
        if (shouldRequestOverlayPermission(context)) {
            Intent(Settings.ACTION_MANAGE_OVERLAY_PERMISSION).apply {
                data = Uri.parse("package:${context.packageName}")
                addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            }.also { intent ->
                context.startActivity(intent)
            }
        } else {
            updateState { it.copy(floatingWindowVisible = true) }
        }
    }

    fun checkOverlayPermission(context: Context) {
        val hasPermission = !shouldRequestOverlayPermission(context)
        updateState { it.copy(floatingWindowVisible = hasPermission) }
    }

    fun cleanUpResources() {
        removeFloatingView()
    }
    // endregion

    // region 内部实现
    private fun fetchGAID() {
        viewModelScope.launch {
            _state.update { it.copy(isLoading = true) }
            try {
                val gaid = gaidHelper.getAdvertisingId()
                _state.update {
                    it.copy(
                        isLoading = false,
                        gaid = gaid
                    )
                }
            } catch (e: Exception) {
                Log.e("ViewModel", "GAID fetch failed", e)
                _state.update {
                    it.copy(
                        isLoading = false,
                        errorMessage = "Failed to get advertising ID: ${e.message}"
                    )
                }
            }
        }
    }

    private fun requestAccessibilityPermission() {
        getApplication<Application>().startActivity(
            Intent(Settings.ACTION_ACCESSIBILITY_SETTINGS).apply {
                addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            }
        )
    }

    private fun shouldRequestOverlayPermission(context: Context): Boolean {
        return Build.VERSION.SDK_INT >= Build.VERSION_CODES.M &&
                !Settings.canDrawOverlays(context)
    }

    private fun removeFloatingView() {
        internalState.floatingViewRef?.get()?.let { view ->
            try {
                windowManager.removeView(view)
            } catch (e: Exception) {
                Log.e("ViewModel", "Remove floating view failed", e)
            }
        }
        internalState.floatingViewRef = null
    }

    fun setFloatingView(view: View) {
        internalState.floatingViewRef = WeakReference(view)
    }

    fun getFloatingView(): View? = internalState.floatingViewRef?.get()

    private inline fun updateState(block: (MainViewState) -> MainViewState) {
        _state.update(block)
    }
    // endregion
}

data class MainViewState(
    val isLoading: Boolean = true,
    val gaid: String = "",
    val isServiceReady: Boolean = false,
    val floatingWindowVisible: Boolean? = null,
    val errorMessage: String? = null
)